using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Threading;

namespace Snowflakes
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class Window1 : Window
    {
        Random ranNum;
        Snowflake snowFlakePic;

        // random numbers used to simulate snow falling and moving the snowflakes on the x and y axis
        double randomBlowingNum = 0.0;   // x axis
        double randomFallingNum = SystemParameters.WorkArea.Height; // y axis
        double randomBlowingTime = 0.0; // x axis blowing time
        double randomFallingTime = 0.0; // y axis falling time

        // center x and center y of the users screen
        double centerX = SystemParameters.WorkArea.Width / 4;
        double centerY = SystemParameters.WorkArea.Height / 4;

        int snowFlakePicIndex = 0;

        const Int32 MINBLOWINGTIME = 10;
        const Int32 MAXBLOWINGTIME = 15;
        const Int32 MINFALLINGTIME = 7;
        const Int32 MAXFALLINGTIME = 21;

        const Int32 TIMETOLOADNEXTPIC = 10;
        const Int32 TIMEFORPICTOBEENLARGED = 8;

        // values to fade in / out the snowflake when it is the one and only enlarged / shrinking one
        const double FADEINFILL = 0.6;
        const double FADEOUTFILL = 0.6;
        const double FADEINOTHERFILL = 0.6;
        const double FADEOUTOTHERFILL = 0.6;

        double CANVASSETTOP;
        const double EXTRAMARGINTOFALL = 225;

        SnowflakePicCollection snowflakePicCollection = new SnowflakePicCollection();
        SnowflakeCollection snowflakeCollection = new SnowflakeCollection();

        public Window1()
        {
            InitializeComponent();

            ranNum = new Random();
            CANVASSETTOP = -EXTRAMARGINTOFALL;

            // grab all of the pictures
            LoadPictures();

            // start off a timer to get each picture to enlarge in the center of the screen
            DispatcherTimer timerSnowFlake = new DispatcherTimer();
            timerSnowFlake.Interval = new TimeSpan(0, 0, TIMETOLOADNEXTPIC);
            timerSnowFlake.Tick += new EventHandler(timerSnowFlake_Tick);
            timerSnowFlake.Start();
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            // start the snow falling
            SnowFall();
        }

        void LoadPictures()
        {
            string path = System.Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            string[] files = System.IO.Directory.GetFiles(path, "*.jpg", System.IO.SearchOption.AllDirectories); // find all .jpg pictures in all directories

            // add to the observable collection
            foreach (string file in files)
            {
                snowflakePicCollection.SnowflakePicsCollection.Add(new SnowflakePic(file));
            }

            // add to the main canvas
            foreach (SnowflakePic wedPic in snowflakePicCollection.SnowflakePicsCollection)
            {
                Snowflake snowFlake = new Snowflake(wedPic);
                snowflakeCollection.SnowflakeCol.Add(snowFlake);
                canvasMainContainer.Children.Add(snowFlake);
            }
        }

        private void SnowFall()
        {

            foreach (FrameworkElement framy in canvasMainContainer.Children)
            {
                if (framy.GetType().Equals(typeof(Snowflake)))
                {
                    Snowflake wedPic = (Snowflake)framy;

                    // ofset each y-axis
                    CANVASSETTOP += 0.5;
                    Canvas.SetTop(wedPic, CANVASSETTOP);
                    repeatSnowFall(wedPic);
                }
            }
        }

        void repeatSnowFall(Snowflake wedPic)
        {
            randomBlowingNum = ranNum.Next(0, Convert.ToInt32(SystemParameters.WorkArea.Width));
            randomBlowingTime = ranNum.Next(MINBLOWINGTIME, MAXBLOWINGTIME);
            randomFallingTime = ranNum.Next(MINFALLINGTIME, MAXFALLINGTIME);

            // y axis ... falling
            DoubleAnimation doubleAnimFalling = new DoubleAnimation(randomFallingNum + EXTRAMARGINTOFALL, TimeSpan.FromSeconds(randomFallingTime));
            doubleAnimFalling.Completed += new EventHandler(doubleAnimFalling_Completed);
            doubleAnimFalling.Name = wedPic.ImgName;
            //doubleAnimFalling.RepeatBehavior = RepeatBehavior.Forever;

            // x axis ... blowing in the wind!
            DoubleAnimation doubleAnimBlowing = new DoubleAnimation(randomBlowingNum, TimeSpan.FromSeconds(randomBlowingTime));
            doubleAnimBlowing.Name = wedPic.ImgName;
            doubleAnimBlowing.Completed += new EventHandler(doubleAnimBlowing_Completed);

            // start the animations
            wedPic.TranslateTrans.BeginAnimation(TranslateTransform.YProperty, doubleAnimFalling);
            wedPic.TranslateTrans.BeginAnimation(TranslateTransform.XProperty, doubleAnimBlowing);
        }

        // once the snowflake reached the bottom, start it over
        void doubleAnimFalling_Completed(object sender, EventArgs e)
        {
            Clock clk = (Clock)sender;
            DoubleAnimation doubleAnimFalling = (DoubleAnimation)clk.Timeline;

            foreach (Snowflake wedPic in snowflakeCollection.SnowflakeCol)
            {
                if (wedPic.ImgName.Equals(doubleAnimFalling.Name))
                {
                    if (!wedPic.IsEnlarged)
                    {
                        wedPic.TranslateTrans.BeginAnimation(TranslateTransform.YProperty, null);   // clear the transform
                        doubleAnimFalling = new DoubleAnimation(randomFallingNum + EXTRAMARGINTOFALL, TimeSpan.FromSeconds(randomFallingTime));
                        doubleAnimFalling.Completed += new EventHandler(doubleAnimFalling_Completed);
                        doubleAnimFalling.Name = wedPic.ImgName;
                        wedPic.TranslateTrans.BeginAnimation(TranslateTransform.YProperty, doubleAnimFalling);
                    }
                }

            }
        }

        void doubleAnimBlowing_Completed(object sender, EventArgs e)
        {
            // get the clock of the completed animation
            Clock clk = (Clock)sender;
            DoubleAnimation doubleAnimBlowingNew = (DoubleAnimation)clk.Timeline;

            // find out which wedPic had its animation completed
            foreach (Snowflake wedPic in snowflakeCollection.SnowflakeCol)
            {
                if (wedPic.ImgName.Equals(doubleAnimBlowingNew.Name))
                {
                    // make a new random animation for the x axis and start it
                    randomBlowingNum = ranNum.Next(0, Convert.ToInt32(SystemParameters.WorkArea.Width));
                    randomBlowingTime = ranNum.Next(MINBLOWINGTIME, MAXBLOWINGTIME);
                    doubleAnimBlowingNew = new DoubleAnimation(randomBlowingNum, TimeSpan.FromSeconds(randomBlowingTime));
                    doubleAnimBlowingNew.Name = wedPic.ImgName;
                    doubleAnimBlowingNew.Completed += new EventHandler(doubleAnimBlowing_Completed);
                    wedPic.TranslateTrans.BeginAnimation(TranslateTransform.XProperty, doubleAnimBlowingNew);
                }
            }
        }

        void timerSnowFlake_Tick(object sender, EventArgs e)
        {
            // get each snowflake picture, 1 at a time (everytime the timer elaspes) and display it enlarged
            snowFlakePic = snowflakeCollection.SnowflakeCol[snowFlakePicIndex];

            DoubleAnimation doubleAnimEnlargeCenter = new DoubleAnimation(0.5, TimeSpan.FromSeconds(0));
            DoubleAnimation doubleAnimEnlarge = new DoubleAnimation(1, TimeSpan.FromSeconds(5));
            DoubleAnimation doubleAnimCenterX = new DoubleAnimation(centerX, TimeSpan.FromSeconds(5));
            DoubleAnimation doubleAnimCenterY = new DoubleAnimation(centerY, TimeSpan.FromSeconds(5));

            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.CenterXProperty, doubleAnimEnlargeCenter);
            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.CenterYProperty, doubleAnimEnlargeCenter);
            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.ScaleXProperty, doubleAnimEnlarge);
            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.ScaleYProperty, doubleAnimEnlarge);

            snowFlakePic.TranslateTransX = snowFlakePic.TranslateTrans.X;
            snowFlakePic.TranslateTransY = snowFlakePic.TranslateTrans.Y;

            // bring the enlarging snowflake to the front
            snowFlakePic.ZIndex = Canvas.GetZIndex(snowFlakePic);
            Canvas.SetZIndex(snowFlakePic, snowflakeCollection.SnowflakeCol.Count + 1);
            snowFlakePic.IsEnlarged = true;

            // fade out the fill
            DoubleAnimation doubleAnimFadeOutSnowflake = new DoubleAnimation(0.0, TimeSpan.FromSeconds(FADEOUTFILL));
            snowFlakePic.ellipseBackground.BeginAnimation(OpacityProperty, doubleAnimFadeOutSnowflake);

            // fade in the other path
            DoubleAnimation doubleAnimFadeInOtherPath = new DoubleAnimation(1.0, TimeSpan.FromSeconds(FADEINOTHERFILL));
            snowFlakePic.pathEnlargePic.BeginAnimation(OpacityProperty, doubleAnimFadeInOtherPath);


            snowFlakePic.TranslateTrans.BeginAnimation(TranslateTransform.XProperty, doubleAnimCenterX);
            snowFlakePic.TranslateTrans.BeginAnimation(TranslateTransform.YProperty, doubleAnimCenterY);

            // if we are at the end of the collection, start all over
            if (snowFlakePicIndex.Equals(snowflakeCollection.SnowflakeCol.Count - 1))
                snowFlakePicIndex = -1;

            snowFlakePicIndex++;

            DispatcherTimer timerSnowFlakeEnlarged = new DispatcherTimer();
            timerSnowFlakeEnlarged.Interval = new TimeSpan(0, 0, TIMEFORPICTOBEENLARGED);
            timerSnowFlakeEnlarged.Tick += new EventHandler(timerSnowFlakeEnlarged_Tick);
            timerSnowFlakeEnlarged.Start();
        }

        void timerSnowFlakeEnlarged_Tick(object sender, EventArgs e)
        {
            // shrink the enlarged snowflake down
            DispatcherTimer snowFlakeTimer = (DispatcherTimer)sender;
            snowFlakeTimer.Stop();

            DoubleAnimation doubleAnimShrinkCenter = new DoubleAnimation(0.5, TimeSpan.FromSeconds(0));
            DoubleAnimation doubleAnimShrink = new DoubleAnimation(0.25, TimeSpan.FromSeconds(2));
            DoubleAnimation doubleAnimOldPosX = new DoubleAnimation(snowFlakePic.TranslateTransX, TimeSpan.FromSeconds(2));
            DoubleAnimation doubleAnimOldPosY = new DoubleAnimation(snowFlakePic.TranslateTransY, TimeSpan.FromSeconds(2));

            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.CenterXProperty, doubleAnimShrinkCenter);
            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.CenterYProperty, doubleAnimShrinkCenter);
            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.ScaleXProperty, doubleAnimShrink);
            snowFlakePic.ScaleTrans.BeginAnimation(ScaleTransform.ScaleYProperty, doubleAnimShrink);

            // reset its zIndex
            Canvas.SetZIndex(snowFlakePic, snowFlakePic.ZIndex);
            snowFlakePic.IsEnlarged = false;

            // fade in the fill
            DoubleAnimation doubleAnimFadeInSnowflake = new DoubleAnimation(1.0, TimeSpan.FromSeconds(FADEINFILL));
            snowFlakePic.ellipseBackground.BeginAnimation(OpacityProperty, doubleAnimFadeInSnowflake);

            // fade out the other path
            DoubleAnimation doubleAnimFadeInOtherPath = new DoubleAnimation(0.0, TimeSpan.FromSeconds(FADEOUTOTHERFILL));
            snowFlakePic.pathEnlargePic.BeginAnimation(OpacityProperty, doubleAnimFadeInOtherPath);

            snowFlakePic.TranslateTrans.BeginAnimation(TranslateTransform.XProperty, doubleAnimOldPosX);
            snowFlakePic.TranslateTrans.BeginAnimation(TranslateTransform.YProperty, doubleAnimOldPosY);

            repeatSnowFall(snowFlakePic);
        }

        void buttonClose_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

    }
}